Package: fastglm
Type: Package
Title: Fast and Stable Fitting of Generalized Linear Models using
        'RcppEigen'
Version: 0.1.1
Authors@R: c(
    person("Jared", "Huling", , email = "jaredhuling@gmail.com", c("aut", "cre")),
    person("Douglas", "Bates", , , c("cph")),
    person("Dirk", "Eddelbuettel", , , c("cph")),
    person("Romain", "Francois", , , c("cph")),
    person("Yixuan", "Qiu", , , c("cph")),
    person("Noah", "Greifer", role=c("ctb"), 
           email = "noah.greifer@gmail.com",
	       comment=c(ORCID="0000-0003-3067-7154"))
    )
Description: Fits generalized linear models efficiently using 'RcppEigen'. The iteratively reweighted least squares 
    implementation utilizes the step-halving approach of Marschner (2011) <doi:10.32614/RJ-2011-012> to help safeguard
    against convergence issues.
BugReports: https://github.com/jaredhuling/fastglm/issues
URL: https://github.com/jaredhuling/fastglm,
        https://jaredhuling.org/fastglm/
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 4.0.0)
Imports: Rcpp (>= 0.12.13), bigmemory, methods, Matrix, Formula
LinkingTo: Rcpp, RcppEigen, BH, bigmemory
Suggests: glm2, knitr, rmarkdown, testthat (>= 3.0.0), sandwich,
        microbenchmark, MASS, statmod, logistf, brglm2, pscl, speedglm,
        tweedie, ggplot2
Config/testthat/edition: 3
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-06-05 13:55:11 UTC; huling
Author: Jared Huling [aut, cre],
  Douglas Bates [cph],
  Dirk Eddelbuettel [cph],
  Romain Francois [cph],
  Yixuan Qiu [cph],
  Noah Greifer [ctb] (ORCID: <https://orcid.org/0000-0003-3067-7154>)
Maintainer: Jared Huling <jaredhuling@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-07 14:40:07 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-06-07 15:00:52 UTC; unix
Archs: fastglm.so.dSYM
