# Generated by roxygen2: do not edit by hand

S3method(coef,fastglm_hurdle)
S3method(coef,fastglm_zi)
S3method(deviance,fastglm)
S3method(family,fastglm)
S3method(fastglm,default)
S3method(logLik,fastglm)
S3method(logLik,fastglm_hurdle)
S3method(logLik,fastglm_nb)
S3method(logLik,fastglm_zi)
S3method(predict,fastglm)
S3method(print,fastglm)
S3method(print,fastglm_hurdle)
S3method(print,fastglm_zi)
S3method(residuals,fastglm)
S3method(sandwich::bread,fastglm)
S3method(sandwich::bread,fastglmFit)
S3method(sandwich::estfun,fastglm)
S3method(sandwich::estfun,fastglmFit)
S3method(sandwich::vcovHC,fastglm)
S3method(sandwich::vcovHC,fastglmFit)
S3method(stats::vcov,fastglm)
S3method(stats::vcov,fastglmFit)
S3method(summary,fastglm)
S3method(summary,fastglmFit)
S3method(vcov,fastglm_hurdle)
S3method(vcov,fastglm_zi)
export(fastglm)
export(fastglmPure)
export(fastglm_control)
export(fastglm_fit)
export(fastglm_hurdle)
export(fastglm_nb)
export(fastglm_streaming)
export(fastglm_zi)
export(negbin)
import(methods)
import(stats)
importClassesFrom(Matrix,dgCMatrix)
importFrom(Rcpp,evalCpp)
importFrom(bigmemory,as.big.matrix)
importFrom(bigmemory,is.big.matrix)
importFrom(methods,new)
useDynLib(fastglm, .registration = TRUE)
