% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastglm-methods.R
\name{vcov.fastglm}
\alias{vcov.fastglm}
\title{`vcov()` method for `fastglm` fitted objects}
\usage{
\method{vcov}{fastglm}(object, ...)
}
\arguments{
\item{object}{a fitted object of class inheriting from `"fastglm"`.}

\item{...}{additional arguments (currently unused).}
}
\value{
The estimated variance-covariance matrix of the fitted coefficients.
For rank-deficient fits, rows and columns corresponding to aliased
coefficients are filled with `NA`.
}
\description{
`vcov()` method for `fastglm` fitted objects
}
